/**
 * 
 */
package simpleDB;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

/**
 * Manager for database connection. In order to allow embedded calls for
 * database methods...
 * 
 * @author rosmord
 */
public class ConnectionManager {

	public static final String MYSQL = "mysql";

	public static final String HSQL= "hsqldb:hsql";
	
	private String server;

	private String databaseName;

	private String login;

	private String password;

	private Connection connection;

	private String baseType;

	/**
	 * Number of embedded database connexions. level.
	 */
	private int depth = 0;

	static {
		loadDrivers();
	}

	public ConnectionManager(String baseType, String server,
			String databaseName, String login, String password) {
		super();
		this.baseType = baseType;
		this.server = server;
		this.login = login;
		this.databaseName = databaseName;
		this.password = password;
		this.depth = 0;
	}


	/**
	 * Loads the correct driver.
	 */
	private static void loadDrivers() {
		String[] drivers = { "com.mysql.jdbc.Driver", "org.hsqldb.jdbcDriver"};
		for (String d : drivers) {
			try {
				Class.forName(d);
			} catch (ClassNotFoundException e) {
				// This kind of errors is expected and normal
				System.err.println(e.getLocalizedMessage());
			}
		}
	}

	/**
	 * Open a connection to the database, if needed.
	 * May be nested.
	 */
	public void open() {
		try {
			if (depth == 0) {
				connection = DriverManager.getConnection(buildURL(), login,
						password);
			}
			depth++;
		} catch (SQLException e) {
			System.err.println("Can't connect : " + buildURL() + " " + login
					+ " " + password);
			throw new PersistenceException("Problem connecting to database : "
					+ e.getMessage());
		}
	}

	/**
	 * Build an url suitable for connecting to the database.
	 * @return
	 */
	private String buildURL() {
		return "jdbc:" + baseType + "://" + server + "/" + databaseName;
	}

	public void close() {
		try {
			depth--;
			if (depth == 0 && connection != null)
				connection.close();
		} catch (SQLException e) {
			throw new RuntimeException(e);
		} finally {
			if (depth == 0)
				connection = null;
		}
	}

	/**
	 * @return the current connection, if it is opened.
	 */
	public Connection getConnection() {
		if (connection == null) {
			throw new RuntimeException("You must open the connection  !");
		}
		return connection;
	}

	/**
	 * @return the databaseName
	 */
	public String getDatabaseName() {
		return databaseName;
	}

	public String getState() {
		return "*** connection depth " + depth;
	}

}
