/**
 * 
 */
package simpleDB;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import simpleDB.utils.ModelConstants;
import simpleDB.utils.PossiblyNullValue;
import simpleDB.utils.ResultSetExtractor;

/**
 * Base class for all DAOs.
 * @author rosmord
 *
 */
public abstract class DAOHelper {


	protected ConnectionManager connectionManager;
	private boolean debug= false;
	

	/**
	 * @param connectionManager2
	 */
	public DAOHelper(ConnectionManager connectionManager2) {
		this.connectionManager= connectionManager2;
	}


	/**
	 * Prepares a suitable indentifier object for injecting into SQL methods.
	 * The idea is that if the current id is not defined, NULL will be passed. 
	 * @param id : the current id for the object to save
	 * @return an object which is either the id, or a representation of SQL NULL.
	 * @see ModelConstants#UNDEFINED_ID
	 */
	protected Object buildIdObject(int id) {
		Object result = new PossiblyNullValue(java.sql.Types.INTEGER);
	
		if (id != ModelConstants.UNDEFINED_ID)
			result = new Integer(id);
		return result;
	}
	
	/**
	 * Helper method for simple SQL queries.
	 * @param sql
	 * @param args
	 * @return
	 * @throws SQLException
	 */
	protected int doUpdate(String sql, Object ... args) throws SQLException {
		int result= 0;
		CloseManager closeManager= new CloseManager();
		try {
			PreparedStatement pst= connectionManager.getConnection().prepareStatement(sql);
			closeManager.manage(pst);
			fillArgs(pst, args);
			result= pst.executeUpdate();
		} finally {
			closeManager.close();
		}
		return result;
	}

	/**
	 * Find a collections of objects.
	 * <p> If the objects class extends ModelObject, the dirty flags will be cleared. 
	 * @param <T2>
	 * @param sql
	 * @param extractor
	 * @param args
	 * @return
	 * @throws SQLException
	 */
	protected<T2> List<T2> findAll(String sql, ResultSetExtractor<? extends T2> extractor, Object ... args) throws SQLException {
		ArrayList<T2> result= new ArrayList<T2>();
		CloseManager closeManager= new CloseManager();
		try {
			PreparedStatement pst= connectionManager.getConnection().prepareStatement(sql);
			closeManager.manage(pst);
			
			fillArgs(pst, args);
			
			ResultSet rs= pst.executeQuery();
			closeManager.manage(rs);
			if (debug)
				System.err.println("starting extracting "+ sql + args);
			while (rs.next()) {
				result.add(extractor.extractFrom(rs));
			}
			if (debug)
				System.err.println("finishing extracting "+ sql + args);
		} finally {
			closeManager.close();
		}
		
		return result;
	}
	/**
	 * Returns a unique object.
	 * If the object class extends ModelObject, its dirty flag will be cleared.
	 * @param sql the SQL request, which should return only one line (or zero).
	 * @param extractor : an extractor for the result type
	 * @param args
	 * @return an object, or null if none is found.
	 * @throws SQLException
	 */
	protected<T> T findObject(String sql, ResultSetExtractor<? extends T> extractor, Object ... args) throws SQLException {
		T result= null;
		CloseManager closeManager= new CloseManager();
		try {
			PreparedStatement pst= connectionManager.getConnection().prepareStatement(sql);
			closeManager.manage(pst);
			
			fillArgs(pst, args);
			
			ResultSet rs= pst.executeQuery();
			closeManager.manage(rs);
			
			if (rs.next()) {
				result= extractor.extractFrom(rs);
			}
		} finally {
			closeManager.close();
		}
		return result;
	}
	
	/**
	 * Fills a prepared statement args from a lists of objects.
	 * @param pst
	 * @param args
	 * @throws SQLException
	 */
	private void fillArgs(PreparedStatement pst, Object... args) throws SQLException {
		for (int i= 0; i < args.length; i++) {
			Object val= args[i];
			
			// setObject(i, null) doesn't always function.
			// so we use PossiblyNullValue to encapsulate those values.
			if (val instanceof PossiblyNullValue) {
				PossiblyNullValue nullValue = (PossiblyNullValue) val;
				if (nullValue.getValue() != null) {
					pst.setObject(i+1, nullValue.getValue());
				} else {
					pst.setNull(i+1, nullValue.getType());
				}
			}  else 
				pst.setObject(i+1, val);
		}
	}
}
